PROJECT_USER_FRAGMENT = '''
id
activated
role
user {
  id
  email
}
'''

PROJECT_USER_FRAGMENT_WITH_KPIS = '''
id
activated
consensusMark
honeypotMark
lastLabelingAt
numberOfAnnotations
numberOfLabeledAssets
numberOfLabels
role
starred
totalDuration
user {
  id
  email
}
'''
